/*
Copyright (c) 2009, Dirk Krause
All rights reserved.

Redistribution and use in source and binary forms,
with or without modification, are permitted provided
that the following conditions are met:

* Redistributions of source code must retain the above
  copyright notice, this list of conditions and the
  following disclaimer.
* Redistributions in binary form must reproduce the above 
  opyright notice, this list of conditions and the following
  disclaimer in the documentation and/or other materials
  provided with the distribution.
* Neither the name of the Dirk Krause nor the names of
  contributors may be used to endorse or promote
  products derived from this software without specific
  prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED.
IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
*/

package bmepsgui;

import javax.swing.SwingUtilities;

import dirk_krause.tools.*;

class GuiCreator implements Runnable {
    private String directory = null;

    public GuiCreator(String[] args) {
        if(args.length > 0) {
            directory = args[0];
        }
    }

    public void run() {
        DkTool dkt = new DkTool(BmepsGuiMain.aName, BmepsGuiMain.gName);
        DkGuiTool dkgt = new DkGuiTool(dkt);
        BmepsController bc = new BmepsController(dkt, dkgt);
        String[] t = BmepsGuiMain.def_t;
        String[] my_t = dkt.getLocalizedArray(this, "bmepsgui.txt", "UTF-8");
        if(my_t != null) {
            if(my_t.length >= BmepsGuiMain.def_t.length) {
                t = my_t;
            }
        }
        BmepsWindow bw = new BmepsWindow(dkt, dkgt, bc, t);
        bw.choosePosition();
        bw.setVisible(true);
	if(directory != null) {
	    bw.useNewDirectory(directory);
	}
    }
}


/**
 *
 * @author krause
 */
public class BmepsGuiMain {
    
    /** Application name. */
    public static final String aName = "Bmeps GUI";
    
    /** Application group name. */
    public static final String gName = "DK Tools";
    
    /** Strings used by the program. */
    public static final String[] def_t = {
        /*   0 */ "",
        /*   1 */ "ChooseDirectory.png",
        /*   2 */ "Options.png",
        /*   3 */ "Run.png",
        /*   4 */ "Exit.png",
        /*   5 */ "Directory",
        /*   6 */ "Options",
        /*   7 */ "Run",
        /*   8 */ "Exit",
        /*   9 */ "Choose directory",
        /*  10 */ "Set conversion options",
        /*  11 */ "Run conversion",
        /*  12 */ "Exit the application",
        /*  13 */ "Directory: ",
        /*  14 */ "Ready ",
        /*  15 */ "File: ",
        /*  16 */ "File",
        /*  17 */ "Setup",
        /*  18 */ "GUI",
        /*  19 */ "Set up GUI and appearance",
        /*  20 */ "?",
        /*  21 */ "Documentation",
        /*  22 */ "Web site",
        /*  23 */ "View the locally installed documentation",
        /*  24 */ "Visit the bmeps web site",
        /*  25 */ "Bmeps GUI Options",
        /*  26 */ "OK",
        /*  27 */ "Confirm settings and close this dialog",
        /*  28 */ "Cancel",
        /*  29 */ "Discard settings and close this dialog",
        /*  30 */ "Reload",
        /*  31 */ "Reload program settings into this dialog",
        /*  32 */ "Defaults",
        /*  33 */ "Suggest reasonable defaults",
        /*  34 */ "Predefined configurations",
        /*  35 */ "Full-custom configuration",
        /*  36 */ "Built-in configuration",
        /*  37 */ "Configuration from file",
        /*  38 */ "Use a built-in configuration",
        /*  39 */ "built-in",
        /*  40 */ "Use a configuration from the configuration file",
        /*  41 */ "config-file",
        /*  42 */ "Make mode",
        /*  43 */ "Check file modification timestamps, run conversion only if necessary",
        /*  44 */ "General options",
        /*  45 */ "Draft mode",
        /*  46 */ "Create placeholder for document draft only",
        /*  47 */ "PS/EPS",
        /*  48 */ "PDF",
        /*  49 */ "Bounding Box",
        /*  50 */ "General",
        /*  51 */ "Alpha channel",
        /*  52 */ "Size + resolution",
        /*  53 */ "PS operators",
        /*  54 */ "Compression + encoding",
        /*  55 */ "PS/EPS version: ",
        /*  56 */ "Encapsulated PostScript",
        /*  57 */ "DSC comments",
        /*  58 */ "Colored output",
        /*  59 */ "Image interpolation",
        /*  60 */ "for JPEGs too",
        /*  61 */ "Choose PS format version",
        /*  62 */ "Produce either EPS or PS",
        /*  63 */ "Add DSC comments to output",
        /*  64 */ "Choose colored or greyscaled output",
        /*  65 */ "Enable/disable image interpolation to improve rendering quality",
        /*  66 */ "Enable/disable image interpolation for JPEG images",
        /*  67 */ "Mix against color: ",
        /*  68 */ "Ignore background chunk",
        /*  69 */ "Create image mask",
        /*  70 */ "require alpha>0",
        /*  71 */ "require full alpha",
        /*  72 */ "Use aplpha channel to mix against a background color",
        /*  73 */ "Click here to specify a default background color",
        /*  74 */ "Ignore background chunk in image file, always use default background color",
        /*  75 */ "Use alpha channel to create an image mask",
        /*  76 */ "1 pixel => 1 point",
        /*  77 */ "Use resolution chunk",
        /*  78 */ "Fit to media size: ",
        /*  79 */ "Convert each input file pixel to one PS point",
        /*  80 */ "Use the resolution chunk from input file (if available)",
        /*  81 */ "Fit output to a specified paper size",
        /*  82 */ "Choose a paper size",
        /*  83 */ "Separated dictionary",
        /*  84 */ "\"vmreclaim\"",
        /*  85 */ "\"showpage\"",
        /*  86 */ "Create a separated dictionary (required to use vmreclaim)",
        /*  87 */ "Ask for PS garbage collection after drawing image",
        /*  88 */ "Use showpage operator",
        /*  89 */ "Convert binary data to ASCII: ",
        /*  90 */ "Choose method to convert binary data to ASCII",
        /*  91 */ "Run-length compression",
        /*  92 */ "Allow use of run-length compression",
        /*  93 */ "Multiple data sources",
        /*  94 */ "Allow multiple data sources for red, green and blue",
        /*  95 */ "Flate compression",
        /*  96 */ "Allow the use of flate compression",
        /*  97 */ "Keep DCT encoding for JPEG input",
        /*  98 */ "Use DCT encoded data from JPEG files directly",
        /*  99 */ "Ignore JFIF check result",
        /* 100 */ "Ignore result of bmeps's JFIF check",
        /* 101 */ "Allowed SOF types",
        /* 102 */ "Enable/disable JPEG SOF type",
        /* 103 */ "PS / EPS options",
        /* 104 */ "There are no options to set for this output driver.",
        /* 105 */ "PDF options",
        /* 106 */ "General",
        /* 107 */ "Size + resolution",
        /* 108 */ "Compression + encoding",
        /* 109 */ "Alpha channel",
        /* 110 */ "PDF version: ",
        /* 111 */ "Fit at open: ",
        /* 112 */ "None",
        /* 113 */ "Width",
        /* 114 */ "Height",
        /* 115 */ "Page",
        /* 116 */ "Transfer alpha channel",
        /* 117 */ "Tranfer alpha channel from input file to output file",
        /* 118 */ "Allow PDF page attributes",
        /* 119 */ "Allow the use of PDF page attributes (may cause conflicts with LaTeX packages)",
        /* 120 */ "PS version changed",
        /* 121 */ "PDF version changed",
        /* 122 */ "Base driver: ",
        /* 123 */ "Color PS",
        /* 124 */ "Color PDF,",
        /* 125 */ "Background color",
        /* 126 */ "Failed to run program.",
        /* 127 */ "The program returned a status indicating a problem.",
        /* 128 */ "Version",
        /* 129 */ "Show version and license information",
        /* 130 */ "index.html",
        /* 131 */ "Bmeps GUI documentation",
        /* 132 */ "http://bmeps.sourceforge.net/gui.html",
        /* 133 */ "Failed to show local documentation!",
        /* 134 */ "Error",
	/* 135 */ "Compression predictor: ",
	/* 136 */ "None",
	/* 137 */ "TIFF Sub",
	/* 138 */ "PNG Sub",
	/* 139 */ "PNG Up",
	/* 140 */ "PNG Average",
	/* 141 */ "PNG Paeth",
	/* 142 */ "Choose a predictor for compression",
    };

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        // TODO code application logic here
        SwingUtilities.invokeLater(new GuiCreator(args));
    }

}
