/*
Copyright (c) 2009, Dirk Krause
All rights reserved.

Redistribution and use in source and binary forms,
with or without modification, are permitted provided
that the following conditions are met:

* Redistributions of source code must retain the above
  copyright notice, this list of conditions and the
  following disclaimer.
* Redistributions in binary form must reproduce the above 
  opyright notice, this list of conditions and the following
  disclaimer in the documentation and/or other materials
  provided with the distribution.
* Neither the name of the Dirk Krause nor the names of
  contributors may be used to endorse or promote
  products derived from this software without specific
  prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED.
IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
*/

package dirk_krause.tools;

import java.awt.Dimension;
import java.awt.BorderLayout;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JScrollBar;
import javax.swing.JEditorPane;
import javax.swing.SwingUtilities;
import javax.swing.TransferHandler;

/**
 * Panel to show log messages.
 * @author krause
 */
public class LogPanel extends JPanel {
    /** Editor pane (contents of the scroll pane. */
    private JEditorPane jed = null;

    /** Scroll pane. */
    private JScrollPane jsp = null;

    /** Content types accepted by a JEditorPane. */
    public static final String[] contentTypes = {
        "text/plain",
        "text/html",
    };

    /**
     * Class to modify the log text.
     */
    private class TextChanger implements Runnable {
        String new_text = null;
        boolean flag = false;
        /**
         * Constructor.
         * @param s New *complete* log text.
         * @param b Flag: Text is HTML.
         */
        public TextChanger(String s, boolean b) {
            new_text = s; flag = b;
        }
        /**
         * Do the real replacement.
         */
        @Override
        public void run() {
            setTextFromAwtThread(new_text, flag);
        }
    }

    /**
     * Constructor.
     */
    public LogPanel() {
        jed = new JEditorPane();
        jsp = new JScrollPane(jed);
        setLayout(new BorderLayout());
        add(jsp, BorderLayout.CENTER);
        jsp.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
        jsp.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
        jed.setPreferredSize(new Dimension(400, 60));
    }

    /**
     * Replace current log text by a new text.
     * This funcion must be called from within the AWT thread only!
     * @param s         New text.
     * @param isHtml    Flag: Text is HTML.
     */
    public void setTextFromAwtThread(String s, boolean isHtml) {
        jed.setEditable(true);
        if(isHtml) {
            jed.setContentType(contentTypes[1]);
        } else {
            jed.setContentType(contentTypes[0]);
        }
        jed.setText(s);
        jed.setEditable(false);
        JScrollBar sb = jsp.getVerticalScrollBar();
        sb.setValue(sb.getMaximum() - sb.getVisibleAmount());
        repaint();
    }

    /**
     * Replace current log text by a new text.
     * @param s         The new text.
     * @param isHtml    Flag: Text is HTML.
     */
    public void setText(String s, boolean isHtml) {
        SwingUtilities.invokeLater(new TextChanger(s, isHtml));
    }

    /**
     * Set transfer handler. The same transfer handler is used for
     * the scroll panel and the editor too.
     * @param newHandler    New transfer handler.
     */
    @Override
    public void setTransferHandler(TransferHandler newHandler) {
        super.setTransferHandler(newHandler);
        jsp.setTransferHandler(newHandler);
        jed.setTransferHandler(newHandler);
    }

}
