/**
 * The dirk_krause.tools package contains tool classes for general
 * programming (non-GUI) and GUI programming.
 * <ul>
 * <li><b>DkTool</b><br/>
 * <ul>
 * <li>Application preferences handling:<br/>
 * An application using a DkTool object specifies the application name
 * and the application group name in the DkTool object constructor.<br/>
 * Preferences can be assigned to different scopes: an application,
 * an application group or defaults for all applications and groups not
 * (yet) specifically configured. Preferences can be stored for a user
 * or for the entire system.<br/>
 * Preferences can be retrieved and set in bulk mode (by specifying an
 * array of preference names and an array of values).
 * </li>
 * <li>Localized resource retrieval:<br/>
 * By default the DkTool object starts search for resources in the base
 * directory of the jar file of a specified object. Subdirectories
 * matching la/co and la are inspected first to find a file, la
 * is the 2-letter language code (lower-case), co is the two-letter
 * country code converted to lower case.<br/>
 * An application can add further resource base directories (local file
 * system or URLs) to search for resources.<br/>
 * </li>
 * <li>String modifications, search for a string index in a string array<br/>
 * These function are designed to use localized strings as action
 * commands.
 * </li>
 * <li>Component positioning calculations.
 * </li>
 * </ul>
 * </li>
 * <li>PlatformInformation<br/>
 * The PlatformInformation class runs the jhelp or jhelp.exe program
 * to retrieve information about the current user and the system.<br/>
 * Some information - i.e. the current user name, the host name and
 * the users language - can be obtained from a DkTool object which retrieves
 * this information from system properties instead of running an external
 * program.
 * </li>
 * <li>DirectoryFilter<br/>
 * Filter class for file chooser. Instead of using this class you can set
 * the file selection mode on the JFileChooser.
 * </li>
 * <li>FileNameDropTransferHandler<br/>
 * A general transfer handler for drag and drop accepting file names.
 * This is an abstract class, the detailed behaviour is defined in the
 * subclasses.
 * </li>
 * <li>OneDirectoryDropTransferHandler<br/>
 * A transfer handler for drag and drop accepting exactly one directory
 * name. This is used i.e. in the Bmeps GUI application.
 * </li>
 * <li>ColorShowArea<br/>
 * A rectangle/square component to show the current color in a color
 * setting. Typically you will set up an action listener and an action
 * command to change the color if a user clicks on the ColorShowArea.
 * </li>
 * <li>DkGuiTool<br/>
 * This class retrieves GUI settings from preferences using the DkTool
 * class and changes the settings for the used look-and-feel, the used
 * font families and enforces a minimum font size.<br/>
 * The DkGuiTool object must be created <i>before</i> you build any
 * GUI element.
 * </li>
 * <li>DkFrame<br/>
 * A frame keeping track of its last position and sizes. This information
 * is stored in preferences using a DkTool object.<br/>
 * </li>
 * <li>SizeTracker<br/>
 * This class keeps track of the size of a GUI element and signalizes
 * whether or not it is necessary to call pack() again if the components
 * minimum or preferred size increased.
 * </li>
 * <li>LogPanel<br/>
 * This panel can be used inside a frame or dialog to show log messages,
 * either plain text or HTML.
 * </li>
 * <li>MessageDialog<br/>
 * The message dialog can be used to show text message (i.e. license
 * and version information).<br/>
 * The text is shown in an uneditable editor pane (what a word), you
 * can copy text to other applications.
 * </li>
 * <li>DkGuiDialog<br/>
 * The DkGuiDialog allows a user to set up the configuration used
 * by the DkGuiTool.
 * </li>
 * </ul>
 */

package dirk_krause.tools;



