/*
Copyright (c) 2001-2008, Dirk Krause
All rights reserved.

Redistribution and use in source and binary forms,
with or without modification, are permitted provided
that the following conditions are met:

* Redistributions of source code must retain the above
  copyright notice, this list of conditions and the
  following disclaimer.
* Redistributions in binary form must reproduce the above 
  opyright notice, this list of conditions and the following
  disclaimer in the documentation and/or other materials
  provided with the distribution.
* Neither the name of the Dirk Krause nor the names of
  its contributors may be used to endorse or promote
  products derived from this software without specific
  prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED.
IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
*/

package dirk_krause.jtools;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.NoSuchElementException;

/**
 * This class can be used to transfer log
 * output from external commands in a background
 * thread to a log viewer window in the main thread.
 * 
 * @author krause
 *
 */
public class LogContents
{
	/**
	 * Texts to be used while building the log text.
	 */
	protected String[] theTexts = {
		"\n",
		""
	};
	
	/**
	 * Collection of all input lines already found.
	 */
	protected ArrayList<String>	textLines = null;
	
	/**
	 * Flag to indicate whether or not a program was
	 * run successfully.
	 */
	protected boolean success = true;


	/**
	 * Show contents always or on error only.
	 */
	protected boolean showAlways = false;
	
	/**
	 * Default constructor.
	 *
	 */
	public LogContents()
	{
		textLines = new ArrayList<String>();
	}
	
	/**
	 * Retrieve the contents (a text to show in the
	 * LogWindow class).
	 * 
	 * @return
	 * The text to show.
	 */
	public String getContents()
	{
		String back = null;
		StringBuilder sb = new StringBuilder(theTexts[1]);
		Iterator<String> i = textLines.iterator();
		while(i.hasNext()) {
			try {
				String s = i.next();
				sb.append(s); sb.append(theTexts[0]);
			}
			catch(NoSuchElementException e) {
			}
		}
		back = new String(sb); sb = null;
		return back;
	}
	
	/**
	 * Add a line of text.
	 * 
	 * @param s
	 * The text line to add.
	 */
	public void add(String s)
	{
		textLines.add(s);
	}
	
	/**
	 * Check whether or not all commands have been
	 * executed successfully.
	 * 
	 * @return
	 * The test result.
	 */
	public boolean getSuccess()
	{
		return success;
	}

	public boolean getShowAlways() {
		return showAlways;
	}

	public void setShowAlways(boolean b) {
		showAlways = b;
	}
	
	/**
	 * Cleanup procedure.
	 *
	 */
	public void cleanup()
	{
		textLines = null;
	}
}



// project dklibsj
// @(#)LogContents.java 1.6 06/23/08

