/*
Copyright (c) 2001-2008, Dirk Krause
All rights reserved.

Redistribution and use in source and binary forms,
with or without modification, are permitted provided
that the following conditions are met:

* Redistributions of source code must retain the above
  copyright notice, this list of conditions and the
  following disclaimer.
* Redistributions in binary form must reproduce the above 
  opyright notice, this list of conditions and the following
  disclaimer in the documentation and/or other materials
  provided with the distribution.
* Neither the name of the Dirk Krause nor the names of
  its contributors may be used to endorse or promote
  products derived from this software without specific
  prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED.
IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
*/

package dirk_krause.jtools;

import java.awt.Container;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.awt.Toolkit;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.JButton;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JTextArea;
import javax.swing.JScrollPane;

import dirk_krause.jtools.SelfPositioningDialog;

/**
 * This class shows log output as plain text.
 * 
 * @author krause
 *
 */
public class LogWindow extends SelfPositioningDialog
  implements ActionListener, WindowListener
{
  /**
   * Text fragments for window title...
   */
  public static final String[] theTexts = {
    "Log",
    "",
    "OK",
    "Version and license",
    "Help",
  };

  /**
   * Border between GUI elements.
   */
  protected int b = 5;
  
  /**
   * The text area to show the log output.
   */
  protected JTextArea ta = null;
  
  /**
   * The OK button to close the window.
   */
  protected JButton bOK = null;
  
  /**
   * Constructor.
   * 
   * @param p
   * The parent frame.
   * 
   * @param c
   * A GuiController providing the border setting.
   */
  public LogWindow(JFrame p, GuiController c)
  {
    super(p);
    b = c.getBorder();
    Container cp = getContentPane();
    cp.setLayout(new BoxLayout(cp, BoxLayout.Y_AXIS));
    cp.add(Box.createRigidArea(new Dimension(b, b)));
    JPanel p1 = new JPanel();
    p1.setLayout(new BoxLayout(p1, BoxLayout.X_AXIS));
    p1.add(Box.createRigidArea(new Dimension(b, b)));
    ta = new JTextArea();
    ta.setRows(15);
    ta.setColumns(60);
    ta.setText(theTexts[1]);
    ta.setEditable(false);
    JScrollPane sp = new JScrollPane(ta);
    sp.setVerticalScrollBarPolicy(
      JScrollPane.VERTICAL_SCROLLBAR_ALWAYS
    );
    sp.setHorizontalScrollBarPolicy(
      JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS
    );
    Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
    int parentwidth = p.getWidth();
    int parentheight = p.getHeight();
    int w = (9 * (d.width - parentwidth)) / 10;
    int h = (9 * (d.height - parentheight)) / 10;
    if(w < (d.width / 3)) { w = (d.width / 3); }
    if(h < (d.height / 3)) { h = (d.height / 3); }
    sp.setPreferredSize(new Dimension(w, h));
    sp.setMinimumSize(new Dimension(10, 10));
    p1.add(sp);
    p1.add(Box.createRigidArea(new Dimension(b, b)));
    cp.add(p1);
    cp.add(Box.createRigidArea(new Dimension(b, b)));
    JPanel p2 = new JPanel();
    p2.setLayout(new BoxLayout(p2, BoxLayout.X_AXIS));
    p2.add(Box.createHorizontalGlue());
    bOK = new JButton(theTexts[2]);
    bOK.addActionListener(this);
    p2.add(bOK);
    p2.add(Box.createHorizontalGlue());
    cp.add(p2);
    cp.add(Box.createRigidArea(new Dimension(b, b)));
    pack();
    addWindowListener(this);
  }
  
  /**
   * Set the text to show.
   * 
   * @param t
   * The text to show.
   */
  public void setText(String t)
  {
    ta.setText(t);
    ta.setEditable(false);
  }
  
  /**
   * Show text from a LogContents only if the
   * command did not execute successfully.
   * 
   * @param c
   * The LogContents providing the text to show.
   */
  public void showTextIfNecessary(LogContents c)
  {
    if((!(c.getSuccess())) || (c.getShowAlways())) {
      ta.setText(c.getContents());
      setVisible(true);
    }
  }
  
  /**
   * Cleanup procedure.
   *
   */
  public void cleanup()
  {
    removeWindowListener(this);
    bOK.removeActionListener(this);
    ta = null;
    bOK = null;
    removeAll();
  }
  
  /**
   * Close (hide) window after pressing OK.
   */
  public void actionPerformed(ActionEvent e) {
    setVisible(false);
    ta.setText(theTexts[1]);
  }
  
  /**
   * Close (hide) window on window close event.
   */
  public void windowClosing(WindowEvent e)
  {
    setVisible(false);
    ta.setText(theTexts[1]);
  }

  /** Unused event handler */
  public void windowActivated(WindowEvent e) {}

  /** Unused event handler */
  public void windowClosed(WindowEvent e) {}

  /** Unused event handler */
  public void windowDeactivated(WindowEvent e) {}

  /** Unused event handler */
  public void windowDeiconified(WindowEvent e) {}

  /** Unused event handler */
  public void windowIconified(WindowEvent e) {}

  /** Unused event handler */
  public void windowOpened(WindowEvent e) {}


}

// vim: set ai sw=2 :



// project dklibsj
// @(#)LogWindow.java 1.6 06/23/08

