/*
Copyright (c) 2001-2008, Dirk Krause
All rights reserved.

Redistribution and use in source and binary forms,
with or without modification, are permitted provided
that the following conditions are met:

* Redistributions of source code must retain the above
  copyright notice, this list of conditions and the
  following disclaimer.
* Redistributions in binary form must reproduce the above 
  opyright notice, this list of conditions and the following
  disclaimer in the documentation and/or other materials
  provided with the distribution.
* Neither the name of the Dirk Krause nor the names of
  its contributors may be used to endorse or promote
  products derived from this software without specific
  prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED.
IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
*/

package dirk_krause.jtools;

import java.awt.Rectangle;
import java.awt.Dimension;
import java.awt.Component;
import java.awt.Toolkit;

import javax.swing.JFrame;
import javax.swing.JDialog;



/**
 * Dialog choosing a position automatically near the
 * parent JFrame.
 *
 * @author  krause
 */
public class SelfPositioningDialog extends JDialog
{

  /** Indicator whether or not a position was choosen */
  private boolean positionWasChoosen = false;
  
  /**
   * Find out whether or not the position is known.
   *
   * @return
   * Flag to indicate whether or not a position was choosen.
   */
  protected synchronized boolean getPositionWasChoosen()
  {
    return positionWasChoosen;
  }
  


  /**
   * Set flag for position was choosen.
   *
   * @param b
   * The new flag value.
   */
  protected synchronized void setPositionWasChoosen(boolean b)
  {
    positionWasChoosen = b;
  }
  


  /**
   * Constructor.
   *
   * @param p
   * The parent JFrame.
   */
  public SelfPositioningDialog(JFrame p)
  {
    super(p);
  }



  /**
   * Constructor without parent JFrame.
   */
  public SelfPositioningDialog()
  {
    super();
  }
  


  /**
   * Choose a non-modal position relative to another component.
   *
   * @param c
   * The parent component.
   */
  public void chooseNonmodalPosition(Component c)
  {
    if(!getPositionWasChoosen()) {
      ComponentPositioner.choosePosition(c, this, false);
      setPositionWasChoosen(true);
    }
  }
  


  /**
   * Choose a modal position relative to another component.
   *
   * @param c
   * The parent component.
   */
  public void chooseModalPosition(Component c)
  {
    if(!getPositionWasChoosen()) {
      ComponentPositioner.choosePosition(c, this, true);
      setPositionWasChoosen(true);
    }
  }

}

// vim: set ai sw=2 :



// project dklibsj
// @(#)SelfPositioningDialog.java 1.9 06/23/08

