/*
Copyright (c) 2001-2008, Dirk Krause
All rights reserved.

Redistribution and use in source and binary forms,
with or without modification, are permitted provided
that the following conditions are met:

* Redistributions of source code must retain the above
  copyright notice, this list of conditions and the
  following disclaimer.
* Redistributions in binary form must reproduce the above 
  opyright notice, this list of conditions and the following
  disclaimer in the documentation and/or other materials
  provided with the distribution.
* Neither the name of the Dirk Krause nor the names of
  its contributors may be used to endorse or promote
  products derived from this software without specific
  prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED.
IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
DAMAGE.
*/

package dirk_krause.jtools;

import java.awt.Dimension;
import javax.swing.JComponent;


/**
 * Keep track of a component size.
 * If the preferred or minimum size of a component increases
 * we need to pack() the window again.
 */
public class SizeTracker {
	/**
	 * The largest width we have ever found.
	 */
	protected int w = 0;
	
	/**
	 * The largest height we have ever found.
	 */
	protected int h = 0;
	
	public SizeTracker()
	{
	}
	
	/**
	 * Check whether or not the window/dialog must
	 * be packed again.
	 *
	 * @param c
	 * The component to keep track.
	 *
	 * @return
	 * Flag to indicate whether or not we must invoke pack() again.
	 */
	public boolean mustPackAgain(JComponent c)
	{
		boolean back = false;
		Dimension ps = c.getPreferredSize();
		Dimension ms = c.getMinimumSize();
		if(ps.width > w) {
			w = ps.width; back = true;
		}
		if(ps.height > h) {
			h = ps.height; back = true;
		}
		if(ms.width > w) {
			w = ms.width; back = true;
		}
		if(ms.height > h) {
			h = ms.height; back = true;
		}
		return back;
	}
}



// project dklibsj
// @(#)SizeTracker.java 1.7 06/23/08

